﻿namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc.String {

  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Data;
  using HeuristicLab.Parameters;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class StringConstantErcValue : WeightedErcValueItem<string> {

    private const string ConstantsParameterName = "Constants";

    public StringConstantErcValue() : this(false, 1d) { }

    public StringConstantErcValue(params string[] constants) : this(true, 1d, constants) { }

    public StringConstantErcValue(bool isEnabled, double weight = 1d, params string[] constants) : base(isEnabled, weight) {
      Name = "String constants";
      Parameters.Add(new FixedValueParameter<StringArray>(ConstantsParameterName, new StringArray(constants)));
    }

    [StorableConstructor]
    public StringConstantErcValue(bool deserializing) : base(deserializing) {
    }

    public StringConstantErcValue(StringConstantErcValue origin, Cloner cloner) : base(origin, cloner) {
    }

    public IValueParameter<StringArray> ConstantsParameter
    {
      get { return (IValueParameter<StringArray>)Parameters[ConstantsParameterName]; }
    }

    public StringArray Constants
    {
      get { return ConstantsParameter.Value; }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new StringConstantErcValue(this, cloner);
    }

    public override string GetErcValue(IRandom random) {
      if (Constants.Length == 0) return string.Empty;

      var x = random.Next(Constants.Length);
      return Constants[x];
    }
  }
}
