﻿namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc {
  using System.Collections.Generic;
  using System.Linq;
  using Common;
  using Core;
  using Data;
  using Parameters;
  using Persistence.Default.CompositeSerializers.Storable;
  using Random;

  [StorableClass]
  public abstract class VectorConstantsErcValue<T, TValue> : WeightedErcValueItem<IReadOnlyList<T>>
    where T : struct
    where TValue : ValueTypeArray<T> {
    private const string ConstantsParameterName = "Constants";

    protected VectorConstantsErcValue(params TValue[] arrays) : this(true, 1d, arrays) { }

    protected VectorConstantsErcValue(bool isEnabled, double weight = 1d, params TValue[] arrays) : base(isEnabled, weight) {
      IsEnabled = isEnabled;
      Parameters.Add(new ValueParameter<ItemCollection<TValue>>(ConstantsParameterName, new ItemCollection<TValue>(arrays)));
    }

    [StorableConstructor]
    private VectorConstantsErcValue(bool deserializing) : base(deserializing) { }

    protected VectorConstantsErcValue(VectorConstantsErcValue<T, TValue> origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<ItemCollection<TValue>> ConstantsParameter
    {
      get { return (IValueParameter<ItemCollection<TValue>>)Parameters[ConstantsParameterName]; }
    }

    public IEnumerable<TValue> Constants
    {
      get { return ConstantsParameter.Value; }
    }

    public override IReadOnlyList<T> GetErcValue(IRandom random) {
      // TODO: remove ToList when ValueTypeArray implements IReadOnlyList
      return Constants.SampleRandom(random).ToArray();
    }
  }
}