﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Gets the last item from the type stack
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorLastExpression<T> : StatelessExpression {
    protected bool Eval(IPushStack<List<T>> vectorStack, IPushStack<T> literalStack) {
      if (vectorStack.IsEmpty || vectorStack.Top.Count == 0) return false;

      var vector = vectorStack.Pop();
      literalStack.Push(vector[vector.Count - 1]);
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].LAST", StackTypes.Integer)]
  public class IntegerVectorLastExpression : VectorLastExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].LAST", StackTypes.Float)]
  public class FloatVectorLastExpression : VectorLastExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatVectorStack, interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].LAST", StackTypes.Boolean)]
  public class BooleanVectorLastExpression : VectorLastExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanVectorStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].LAST", StackTypes.String)]
  public class StringVectorLastExpression : VectorLastExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringVectorStack, interpreter.StringStack);
    }
  }
}

