﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {

  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Gets the nth item from the type stack
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorNthExpression<T> : StatelessExpression {
    protected bool Eval(
      IInternalPushInterpreter interpreter,
      IPushStack<List<T>> vectorStack,
      IPushStack<T> literalStack) {
      if (vectorStack.IsEmpty || vectorStack.Top.Count == 0 || interpreter.IntegerStack.IsEmpty) return false;

      var vector = vectorStack.Pop();
      var n = (int)interpreter.IntegerStack.Pop() % vector.Count;

      if (n < 0) n = n + vector.Count;

      literalStack.Push(vector[n]);
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].NTH", StackTypes.Integer)]
  public class IntegerVectorNthExpression : VectorNthExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.IntegerVectorStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].NTH", StackTypes.Float | StackTypes.Integer)]
  public class FloatVectorNthExpression : VectorNthExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.FloatVectorStack, interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].NTH", StackTypes.Boolean | StackTypes.Integer)]
  public class BooleanVectorNthExpression : VectorNthExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.BooleanVectorStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].NTH", StackTypes.String | StackTypes.Integer)]
  public class StringVectorNthExpression : VectorNthExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.StringVectorStack, interpreter.StringStack);
    }
  }
}
