﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Problem {
  using Common;
  using Configuration;
  using Core;
  using Encodings.IntegerVectorEncoding;
  using Expressions;
  using Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class PushSolution : NamedItem {
    [Storable]
    public readonly double Quality;
    [Storable]
    public readonly IntegerVector IntegerVector;
    [Storable]
    public readonly IRandom Random;
    [Storable]
    public readonly IReadOnlyPushConfiguration Config;

    [Storable]
    public readonly IPushEvaluator Evaluator;

    public readonly PushProgram Program;

    public PushSolution(IntegerVector integerVector, double quality, IRandom random, IReadOnlyPushConfiguration config, IPushEvaluator evaluator, bool simplify = false)
      : base("Solution", "A push solution.") {
      IntegerVector = integerVector;
      Quality = quality;
      Random = random;
      Config = config;
      Evaluator = evaluator;

      Program = IntegerVector.ToPushProgram(config);

      if (simplify)
        Program = Simplifier.Simplifier.Simplify(Program, p => Evaluator.EvaluateTraining(config, p, random).TotalQuality);
    }

    public PushSolution(PushSolution origin, Cloner cloner) : base(origin, cloner) {
      Quality = origin.Quality;
      IntegerVector = cloner.Clone(origin.IntegerVector);
      Random = cloner.Clone(origin.Random);
      Config = cloner.Clone(origin.Config);
    }

    public virtual PushSolution Simplify() {
      return new PushSolution(IntegerVector, Quality, Random, Config, Evaluator, true);
    }

    [StorableConstructor]
    protected PushSolution(bool deserializing) : base(deserializing) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PushSolution(this, cloner);
    }
  }
}
